@@ -1,5 +1,5 @@
 /*
- * The ARP Scanner (arp-scan) is Copyright (C) 2005-2011 Roy Hills,
+ * The ARP Scanner (arp-scan) is Copyright (C) 2005-2013 Roy Hills,
  * NTA Monitor Ltd.
  *
  * This file is part of arp-scan.
@@ -17,7 +17,7 @@
  * You should have received a copy of the GNU General Public License
  * along with arp-scan.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: arp-scan.c 18169 2011-04-04 15:33:58Z rsh $
+ * $Id$
  *
  * arp-scan -- The ARP Scanner
  *
@@ -38,7 +38,7 @@
 
 #include "arp-scan.h"
 
-static char const rcsid[] = "$Id: arp-scan.c 18169 2011-04-04 15:33:58Z rsh $";   /* RCS ID for ident(1) */
+static char const rcsid[] = "$Id$";   /* RCS ID for ident(1) */
 
 /* Global variables */
 static host_entry *helist = NULL;	/* Array of host entries */
@@ -49,8 +49,6 @@ static unsigned responders = 0;		/* Number of hosts which responded */
 static unsigned live_count;		/* Number of entries awaiting reply */
 static int verbose=0;			/* Verbose level */
 static int debug = 0;			/* Debug flag */
-static pcap_t *pcap_handle;		/* pcap handle */
-static pcap_dumper_t *pcap_dump_handle = NULL;	/* pcap savefile handle */
 static char filename[MAXLINE];		/* Target list file name */
 static int filename_flag=0;		/* Set if using target list file */
 static int random_flag=0;		/* Randomise the list */
@@ -93,6 +91,8 @@ static int pkt_write_file_flag=0;	/* Write packet to file flag */
 static int pkt_read_file_flag=0;	/* Read packet from file flag */
 static char pkt_filename[MAXLINE];	/* Read/Write packet to file filename */
 static int write_pkt_to_file=0;		/* Write packet to file for debugging */
+static int rtt_flag=0;			/* Display round-trip time */
+static pcap_dumper_t *pcap_dump_handle = NULL;	/* pcap savefile handle */
 
 int
 main(int argc, char *argv[]) {
@@ -119,9 +119,9 @@ main(int argc, char *argv[]) {
    bpf_u_int32 localnet;
    int datalink;
    int get_addr_status = 0;
-   link_t *link_handle;		/* Handle for link-layer functions */
    int pcap_fd;			/* Pcap file descriptor */
    unsigned char interface_mac[ETH_ALEN];
+   pcap_t *pcap_handle;		/* pcap handle */
 /*
  *      Initialise file names to the empty string.
  */
@@ -139,58 +139,55 @@ main(int argc, char *argv[]) {
    Gettimeofday(&start_time);
    if (debug) {print_times(); printf("main: Start\n");}
 /*
- *	Determine network interface to use if not reading from a pcap file
- *	or writing to a binary file.
- *	If the interface was specified with the --interface option then use
- *	that, otherwise use pcap_lookupdev() to pick a suitable interface.
+ *	Obtain network interface details unless we're reading
+ *	from a pcap file or writing to a binary file.
  */
-   if (!if_name && !pkt_read_file_flag && !pkt_write_file_flag) {
+   if (!pkt_read_file_flag && !pkt_write_file_flag) {
+/*
+ *	Determine network interface to use. If the interface was specified
+ *	with the --interface option then use that, otherwise use
+ *	pcap_lookupdev() to pick a suitable interface.
+ */
+   if (!if_name) {
       if (!(if_name=pcap_lookupdev(errbuf))) {
          err_msg("pcap_lookupdev: %s", errbuf);
       }
    }
-/*
- *      Open link layer socket. This is used to send outbound packets.
- *	If we are reading packets from a pcap file, or writing packets to a
- *	binary file, set this to NULL as we don't need to send packets in
- *	this case.
- */
-   if (!pkt_read_file_flag && !pkt_write_file_flag) {
-      if ((link_handle = link_open(if_name)) == NULL) {
-         if (errno == EPERM || errno == EACCES)
-            warn_msg("You need to be root, or arp-scan must be SUID root, "
-                     "to open a link-layer socket.");
-         err_sys("link_open");
-      }
-   } else {	/* Set link_handle to NULL if reading packets from a file */
-      link_handle = NULL;
-   }
 /*
  *	Obtain the MAC address for the selected interface, and use this
  *	as default for the source hardware addresses in the frame header
  *	and ARP packet if the user has not specified their values.
+ *
+ *	Die with an error if we can't get the MAC address, as this
+ *	indicates that the interface doesn't have a MAC address, so is
+ *	probably not a compatible interface type.
  */
-   if (link_handle) {
-      get_hardware_address(link_handle, interface_mac);
+      get_hardware_address(if_name, interface_mac);
+      if (interface_mac[0]==0 && interface_mac[1]==0 &&
+          interface_mac[2]==0 && interface_mac[3]==0 &&
+          interface_mac[4]==0 && interface_mac[5]==0) {
+         err_msg("ERROR: Could not obtain MAC address for interface %s",
+                 if_name);
+      }
       if (source_mac_flag == 0)
          memcpy(source_mac, interface_mac, ETH_ALEN);
       if (arp_sha_flag == 0)
          memcpy(arp_sha, interface_mac, ETH_ALEN);
-   }
 /*
  *	If the user has not specified the ARP source address, obtain the
  *	interface IP address and use that as the default value.
  */
-   if (arp_spa_flag == 0 && link_handle) {
-      get_addr_status = get_source_ip(link_handle, &arp_spa);
-      if (get_addr_status == -1) {
-         warn_msg("WARNING: Could not obtain IP address for interface %s. "
-                  "Using 0.0.0.0 for", if_name);
-         warn_msg("the source address, which is probably not what you want.");
-         warn_msg("Either configure %s with an IP address, or manually specify"
-                  " the address", if_name);
-         warn_msg("with the --arpspa option.");
-         memset(&arp_spa, '\0', sizeof(arp_spa));
+      if (arp_spa_flag == 0) {
+         get_addr_status = get_source_ip(if_name, &arp_spa);
+         if (get_addr_status == -1) {
+            warn_msg("WARNING: Could not obtain IP address for interface %s. "
+                     "Using 0.0.0.0 for", if_name);
+            warn_msg("the source address, which is probably not what you want.");
+            warn_msg("Either configure %s with an IP address, or manually specify"
+                     " the address", if_name);
+            warn_msg("with the --arpspa option.");
+            memset(&arp_spa, '\0', sizeof(arp_spa));
+         }
       }
    }
 /*
@@ -210,7 +207,7 @@ main(int argc, char *argv[]) {
       pcap_handle = NULL;
    }
 /*
- *	If we are reading files with pcap, get and display the datalink details
+ *	If we are reading data with pcap, get and display the datalink details
  */
    if (pcap_handle) {
       if ((datalink=pcap_datalink(pcap_handle)) < 0)
@@ -290,7 +287,7 @@ main(int argc, char *argv[]) {
          warn_msg("DEBUG: pcap filter string: \"%s\"", filter_string);
       if ((pcap_compile(pcap_handle, &filter, filter_string, OPTIMISE,
            netmask)) < 0)
-         err_msg("pcap_geterr: %s", pcap_geterr(pcap_handle));
+         err_msg("pcap_compile: %s", pcap_geterr(pcap_handle));
       free(filter_string);
       if ((pcap_setfilter(pcap_handle, &filter)) < 0)
          err_msg("pcap_setfilter: %s", pcap_geterr(pcap_handle));
@@ -340,30 +337,21 @@ main(int argc, char *argv[]) {
       if ((hash_table = hash_new()) == NULL)
          err_sys("hash_new");
 
-      if (*ouifilename == '\0')	/* If OUI filename not specified */
-         fn = make_message("%s/%s", DATADIR, OUIFILENAME);
-      else
-         fn = make_message("%s", ouifilename);
+      fn = get_mac_vendor_filename(ouifilename, DATADIR, OUIFILENAME);
       count = add_mac_vendor(hash_table, fn);
       if (verbose > 1 && count > 0)
          warn_msg("DEBUG: Loaded %d IEEE OUI/Vendor entries from %s.",
                   count, fn);
       free(fn);
 
-      if (*iabfilename == '\0')	/* If IAB filename not specified */
-         fn = make_message("%s/%s", DATADIR, IABFILENAME);
-      else
-         fn = make_message("%s", iabfilename);
+      fn = get_mac_vendor_filename(iabfilename, DATADIR, IABFILENAME);
       count = add_mac_vendor(hash_table, fn);
       if (verbose > 1 && count > 0)
          warn_msg("DEBUG: Loaded %d IEEE IAB/Vendor entries from %s.",
                   count, fn);
       free(fn);
 
-      if (*macfilename == '\0')	/* If MAC filename not specified */
-         fn = make_message("%s/%s", DATADIR, MACFILENAME);
-      else
-         fn = make_message("%s", macfilename);
+      fn = get_mac_vendor_filename(macfilename, DATADIR, MACFILENAME);
       count = add_mac_vendor(hash_table, fn);
       if (verbose > 1 && count > 0)
          warn_msg("DEBUG: Loaded %d MAC/Vendor entries from %s.",
@@ -586,7 +574,7 @@ main(int argc, char *argv[]) {
             } else {    /* Retry limit not reached for this host */
                if ((*cursor)->num_sent)
                   (*cursor)->timeout *= backoff_factor;
-               send_packet(link_handle, *cursor, &last_packet_time);
+               send_packet(pcap_handle, *cursor, &last_packet_time);
                advance_cursor();
             }
          } else {       /* We can't send a packet to this host yet */
@@ -602,14 +590,12 @@ main(int argc, char *argv[]) {
          select_timeout = req_interval - loop_timediff;
          if (debug) {print_times(); printf("main: Can't send packet yet. loop_timediff=" ARP_UINT64_FORMAT "\n", loop_timediff);}
       } /* End If */
-      recvfrom_wto(pcap_fd, select_timeout);
+      recvfrom_wto(pcap_fd, select_timeout, pcap_handle);
    } /* End While */
 
    printf("\n");        /* Ensure we have a blank line */
 
-   if (link_handle)
-      link_close(link_handle);
-   clean_up();
+   clean_up(pcap_handle);
    if (write_pkt_to_file)
       close(write_pkt_to_file);
 
@@ -637,6 +623,7 @@ main(int argc, char *argv[]) {
  *	framing		Framing type (e.g. Ethernet II, LLC)
  *	vlan_id		802.1Q VLAN identifier, or -1 if not 802.1Q
  *	frame_hdr	The Ethernet frame header
+ *	pcap_header	The PCAP header struct
  *
  *      Returns:
  *
@@ -648,7 +635,8 @@ main(int argc, char *argv[]) {
 void
 display_packet(host_entry *he, arp_ether_ipv4 *arpei,
                const unsigned char *extra_data, size_t extra_data_len,
-               int framing, int vlan_id, ether_hdr *frame_hdr) {
+               int framing, int vlan_id, ether_hdr *frame_hdr,
+               const struct pcap_pkthdr *pcap_header) {
    char *msg;
    char *cp;
    char *cp2;
@@ -763,6 +751,27 @@ display_packet(host_entry *he, arp_ether_ipv4 *arpei,
          msg = make_message("%s (DUP: %u)", cp, he->num_recv);
          free(cp);
       }
+/*
+ *	If the rtt_flag is set, calculate and report the packet round-trip
+ *	time.
+ */
+      if (rtt_flag) {
+         struct timeval rtt;
+         struct timeval pcap_timestamp;
+         unsigned long rtt_us; /* round-trip time in microseconds */
+/*
+ * We can't pass a pointer to pcap_header->ts directly to timeval_diff
+ * because it's not guaranteed to have the same size as a struct timeval.
+ * E.g. OpenBSD 5.1 on amd64.
+ */
+         pcap_timestamp.tv_sec = pcap_header->ts.tv_sec;
+         pcap_timestamp.tv_usec = pcap_header->ts.tv_usec;
+         timeval_diff(&pcap_timestamp, &(he->last_send_time), &rtt);
+         rtt_us = rtt.tv_sec * 1000000 + rtt.tv_usec;
+         cp=msg;
+         msg=make_message("%s\tRTT=%lu.%03lu ms", cp, rtt_us/1000, rtt_us%1000);
+         free(cp);
+      }
    }	/* End if (!quiet_flag) */
 /*
  *	Print the message.
@@ -776,7 +785,7 @@ display_packet(host_entry *he, arp_ether_ipv4 *arpei,
  *
  *	Inputs:
  *
- *	link_handle		Link layer handle
+ *	pcap_handle	Pcap handle
  *	he		Host entry to send to. If NULL, then no packet is sent
  *	last_packet_time	Time when last packet was sent
  *
@@ -788,11 +797,15 @@ display_packet(host_entry *he, arp_ether_ipv4 *arpei,
  *      identified by "he" using the socket "s". It also updates the
  *	"last_send_time" field for the host entry.
  *
- *	If link_handle is NULL, then don't attempt to send any packets. This
- *	is typically when we are reading packets from a pcap file.
+ *	If we are using the undocumented --writepkttofile option, then we
+ *	write the packet to the write_pkt_to_file file descriptor instead of
+ *	transmitting it on the network.
+ *
+ *	If we are using the undocumented --readpktfromfile option, then we
+ *	don't send anything.
  */
 int
-send_packet(link_t *link_handle, host_entry *he,
+send_packet(pcap_t *pcap_handle, host_entry *he,
             struct timeval *last_packet_time) {
    unsigned char buf[MAX_FRAME];
    size_t buflen;
@@ -858,10 +871,8 @@ send_packet(link_t *link_handle, host_entry *he,
                my_ntoa(he->addr), he->timeout);
    if (write_pkt_to_file) {
       nsent = write(write_pkt_to_file, buf, buflen);
-   } else {
-      if (link_handle) {	/* Don't send if reading packets from file */
-         nsent = link_send(link_handle, buf, buflen);
-      }
+   } else if (!pkt_read_file_flag) {
+      nsent = pcap_sendpacket(pcap_handle, buf, buflen);
    }
    if (nsent < 0)
       err_sys("ERROR: failed to send packet");
@@ -885,7 +896,7 @@ send_packet(link_t *link_handle, host_entry *he,
  *      It does not have to do anything.
  */
 void
-clean_up(void) {
+clean_up(pcap_t *pcap_handle) {
    struct pcap_stat stats;
 
    if (pcap_handle && !pkt_read_file_flag) {
@@ -1044,15 +1055,21 @@ usage(int status, int detailed) {
       fprintf(stderr, "\n--ignoredups or -g\tDon't display duplicate packets.\n");
       fprintf(stderr, "\t\t\tBy default, duplicate packets are displayed and are\n");
       fprintf(stderr, "\t\t\tflagged with \"(DUP: n)\".\n");
-      fprintf(stderr, "\n--ouifile=<s> or -O <s>\tUse OUI file <s>, default=%s/%s\n", DATADIR, OUIFILENAME);
-      fprintf(stderr, "\t\t\tThis file provides the IEEE Ethernet OUI to vendor\n");
-      fprintf(stderr, "\t\t\tstring mapping.\n");
-      fprintf(stderr, "\n--iabfile=<s> or -F <s>\tUse IAB file <s>, default=%s/%s\n", DATADIR, IABFILENAME);
-      fprintf(stderr, "\t\t\tThis file provides the IEEE Ethernet IAB to vendor\n");
-      fprintf(stderr, "\t\t\tstring mapping.\n");
-      fprintf(stderr, "\n--macfile=<s> or -m <s>\tUse MAC/Vendor file <s>, default=%s/%s\n", DATADIR, MACFILENAME);
-      fprintf(stderr, "\t\t\tThis file provides the custom Ethernet MAC to vendor\n");
-      fprintf(stderr, "\t\t\tstring mapping.\n");
+      fprintf(stderr, "\n--ouifile=<s> or -O <s>\tUse IEEE Ethernet OUI to vendor mapping file <s>.\n");
+      fprintf(stderr, "\t\t\tIf this option is not specified, the default filename\n");
+      fprintf(stderr, "\t\t\tis %s in the current directory. If that is\n", OUIFILENAME);
+      fprintf(stderr, "\t\t\tnot found, then the file\n");
+      fprintf(stderr, "\t\t\t%s/%s is used.\n", DATADIR, OUIFILENAME);
+      fprintf(stderr, "\n--iabfile=<s> or -O <s>\tUse IEEE Ethernet IAB to vendor mapping file <s>.\n");
+      fprintf(stderr, "\t\t\tIf this option is not specified, the default filename\n");
+      fprintf(stderr, "\t\t\tis %s in the current directory. If that is\n", IABFILENAME);
+      fprintf(stderr, "\t\t\tnot found, then the file\n");
+      fprintf(stderr, "\t\t\t%s/%s is used.\n", DATADIR, IABFILENAME);
+      fprintf(stderr, "\n--macfile=<s> or -O <s>\tUse custom Ethernet MAC to vendor mapping file <s>.\n");
+      fprintf(stderr, "\t\t\tIf this option is not specified, the default filename\n");
+      fprintf(stderr, "\t\t\tis %s in the current directory. If that is\n", MACFILENAME);
+      fprintf(stderr, "\t\t\tnot found, then the file\n");
+      fprintf(stderr, "\t\t\t%s/%s is used.\n", DATADIR, MACFILENAME);
       fprintf(stderr, "\n--srcaddr=<m> or -S <m> Set the source Ethernet MAC address to <m>.\n");
       fprintf(stderr, "\t\t\tThis sets the 48-bit hardware address in the Ethernet\n");
       fprintf(stderr, "\t\t\tframe header for outgoing ARP packets. It does not\n");
@@ -1156,6 +1173,7 @@ usage(int status, int detailed) {
       fprintf(stderr, "\t\t\tand displayed. This savefile can be analysed with\n");
       fprintf(stderr, "\t\t\tprograms that understand the pcap file format, such as\n");
       fprintf(stderr, "\t\t\t\"tcpdump\" and \"wireshark\".\n");
+      fprintf(stderr, "\n--rtt or -D\t\tDisplay the packet round-trip time.\n");
    } else {
       fprintf(stderr, "use \"arp-scan --help\" for detailed information on the available options.\n");
    }
@@ -1217,24 +1235,21 @@ add_host_pattern(const char *pattern, unsigned host_timeout) {
       if ((result=regcomp(&iprange_pat, iprange_pat_str,
                           REG_EXTENDED|REG_NOSUB))) {
          char errbuf[MAXLINE];
-         size_t errlen;
-         errlen=regerror(result, &iprange_pat, errbuf, MAXLINE);
+         regerror(result, &iprange_pat, errbuf, MAXLINE);
          err_msg("ERROR: cannot compile regex pattern \"%s\": %s",
                  iprange_pat_str, errbuf);
       }
       if ((result=regcomp(&ipslash_pat, ipslash_pat_str,
                           REG_EXTENDED|REG_NOSUB))) {
          char errbuf[MAXLINE];
-         size_t errlen;
-         errlen=regerror(result, &ipslash_pat, errbuf, MAXLINE);
+         regerror(result, &ipslash_pat, errbuf, MAXLINE);
          err_msg("ERROR: cannot compile regex pattern \"%s\": %s",
                  ipslash_pat_str, errbuf);
       }
       if ((result=regcomp(&ipmask_pat, ipmask_pat_str,
                           REG_EXTENDED|REG_NOSUB))) {
          char errbuf[MAXLINE];
-         size_t errlen;
-         errlen=regerror(result, &ipmask_pat, errbuf, MAXLINE);
+         regerror(result, &ipmask_pat, errbuf, MAXLINE);
          err_msg("ERROR: cannot compile regex pattern \"%s\": %s",
                  ipmask_pat_str, errbuf);
       }
@@ -1554,8 +1569,9 @@ find_host(host_entry **he, struct in_addr *addr) {
  *
  *	Inputs:
  *
- *	s	= Socket file descriptor.
- *	tmo	= Select timeout in us.
+ *	sock_fd		= Socket file descriptor.
+ *	tmo		= Select timeout in us.
+ *	pcap_handle 	= pcap handle
  *
  *	Returns:
  *
@@ -1566,21 +1582,21 @@ find_host(host_entry **he, struct in_addr *addr) {
  *	device.
  */
 void
-recvfrom_wto(int s, int tmo) {
+recvfrom_wto(int sock_fd, int tmo, pcap_t *pcap_handle) {
    fd_set readset;
    struct timeval to;
    int n;
 
    FD_ZERO(&readset);
-   if (s >= 0)
-      FD_SET(s, &readset);
+   if (sock_fd >= 0)
+      FD_SET(sock_fd, &readset);
    to.tv_sec  = tmo/1000000;
    to.tv_usec = (tmo - 1000000*to.tv_sec);
-   n = select(s+1, &readset, NULL, NULL, &to);
+   n = select(sock_fd+1, &readset, NULL, NULL, &to);
    if (debug) {print_times(); printf("recvfrom_wto: select end, tmo=%d, n=%d\n", tmo, n);}
    if (n < 0) {
       err_sys("select");
-   } else if (n == 0 && s >= 0) {
+   } else if (n == 0 && sock_fd >= 0) {
 /*
  * For the BPF pcap implementation, we call pcap_dispatch() even if select
  * times out. This is because on many BPF implementations, select() doesn't
@@ -1690,7 +1706,7 @@ callback(u_char *args ATTRIBUTE_UNUSED,
             pcap_dump((unsigned char *)pcap_dump_handle, header, packet_in);
          }
          display_packet(temp_cursor, &arpei, extra_data, extra_data_len,
-                        framing, vlan_id, &frame_hdr);
+                        framing, vlan_id, &frame_hdr, header);
          responders++;
       }
       if (verbose > 1)
@@ -1759,10 +1775,11 @@ process_options(int argc, char *argv[]) {
       {"pcapsavefile", required_argument, 0, 'W'},
       {"writepkttofile", required_argument, 0, OPT_WRITEPKTTOFILE},
       {"readpktfromfile", required_argument, 0, OPT_READPKTFROMFILE},
+      {"rtt", no_argument, 0, 'D'},
       {0, 0, 0, 0}
    };
    const char *short_options =
-      "f:hr:t:i:b:vVdn:I:qgRNB:O:s:o:H:p:T:P:a:A:y:u:w:S:F:m:lLQ:W:";
+      "f:hr:t:i:b:vVdn:I:qgRNB:O:s:o:H:p:T:P:a:A:y:u:w:S:F:m:lLQ:W:D";
    int arg;
    int options_index=0;
 
@@ -1905,6 +1922,9 @@ process_options(int argc, char *argv[]) {
             strlcpy(pkt_filename, optarg, sizeof(pkt_filename));
             pkt_read_file_flag=1;
             break;
+         case 'D':	/* --rtt */
+            rtt_flag = 1;
+            break;
          default:	/* Unknown option */
             usage(EXIT_FAILURE, 0);
             break;	/* NOTREACHED */
@@ -1928,7 +1948,7 @@ process_options(int argc, char *argv[]) {
 void
 arp_scan_version (void) {
    fprintf(stderr, "%s\n\n", PACKAGE_STRING);
-   fprintf(stderr, "Copyright (C) 2005-2011 Roy Hills, NTA Monitor Ltd.\n");
+   fprintf(stderr, "Copyright (C) 2005-2013 Roy Hills, NTA Monitor Ltd.\n");
    fprintf(stderr, "arp-scan comes with NO WARRANTY to the extent permitted by law.\n");
    fprintf(stderr, "You may redistribute copies of arp-scan under the terms of the GNU\n");
    fprintf(stderr, "General Public License.\n");
@@ -2268,8 +2288,7 @@ add_mac_vendor(struct hash_control *table, const char *map_filename) {
       first_call=0;
       if ((result=regcomp(&oui_pat, oui_pat_str, REG_EXTENDED))) {
          char reg_errbuf[MAXLINE];
-         size_t errlen;
-         errlen=regerror(result, &oui_pat, reg_errbuf, MAXLINE);
+         regerror(result, &oui_pat, reg_errbuf, MAXLINE);
          err_msg("ERROR: cannot compile regex pattern \"%s\": %s",
                  oui_pat_str, reg_errbuf);
       }
@@ -2293,8 +2312,7 @@ add_mac_vendor(struct hash_control *table, const char *map_filename) {
          warn_msg("WARNING: Could not parse oui: %s", line);
       } else if (result != 0) {
          char reg_errbuf[MAXLINE];
-         size_t errlen;
-         errlen=regerror(result, &oui_pat, reg_errbuf, MAXLINE);
+         regerror(result, &oui_pat, reg_errbuf, MAXLINE);
          err_msg("ERROR: oui regexec failed: %s", reg_errbuf);
       } else {
          key_len = pmatch[1].rm_eo - pmatch[1].rm_so;
@@ -2323,3 +2341,95 @@ add_mac_vendor(struct hash_control *table, const char *map_filename) {
    fclose(fp);
    return line_count;
 }
+
+/*
+ *	get_mac_vendor_filename -- Determine MAC/Vendor mapping filename
+ *
+ *	Inputs:
+ *
+ *	specified_filename	The filename specified on the command line
+ *	default_datadir		The default data directory
+ *	default_filename	The default filename
+ *
+ *	Returns:
+ *
+ *	The MAC/Vendor mapping filename.
+ *
+ *	If a filename was specified as an option on the command line, then
+ *	that filename is used. Otherwise we look for the default filename
+ *	in the current directory, and use that if it's present. Otherwise
+ *	we use the default filename in the default directory.
+ *
+ */
+char *
+get_mac_vendor_filename(const char *specified_filename,
+                        const char *default_datadir,
+                        const char *default_filename) {
+   struct stat statbuf;
+   int status;
+   char *file_name;
+
+   if (*specified_filename == '\0') {	/* No filename specified */
+      file_name = make_message("%s", default_filename);
+      status = stat(file_name, &statbuf);
+      if (status == -1 && errno == ENOENT) {
+         free(file_name);
+         file_name = make_message("%s/%s", default_datadir, default_filename);
+      }
+   } else {	/* Filename specified */
+      file_name = make_message("%s", specified_filename);
+   }
+   return file_name;
+}
+
+/*
+ *      get_source_ip   -- Get IP address associated with given interface
+ *
+ *      Inputs:
+ *
+ *      interface_name  The name of the network interface
+ *      ip_addr         (output) The IP Address associated with the device
+ *
+ *      Returns:
+ *
+ *      Zero on success, or -1 on failure.
+ */
+int
+get_source_ip(const char *interface_name, uint32_t *ip_addr) {
+   char errbuf[PCAP_ERRBUF_SIZE];
+   pcap_if_t *alldevsp;
+   pcap_if_t *device;
+   pcap_addr_t *addr;
+   struct sockaddr *sa;
+   struct sockaddr_in *sin = NULL;
+
+   if ((pcap_findalldevs(&alldevsp, errbuf)) != 0) {
+      printf("pcap_findalldevs: %s\n", errbuf);
+   }
+
+   device=alldevsp;
+   while (device != NULL && (strcmp(device->name,interface_name) != 0)) {
+      device=device->next;
+   }
+   if (device == NULL) {
+      warn_msg("ERROR: Could not find interface: %s", interface_name);
+      err_msg("ERROR: Check that the interface exists and is up");
+   }
+
+   for (addr=device->addresses; addr != NULL; addr=addr->next) {
+      sa = addr->addr;
+      if (sa->sa_family == AF_INET) {
+         sin = (struct sockaddr_in *) sa;
+         break;
+      }
+   }
+   if (sin == NULL) {
+      return -1;
+   }
+
+   memcpy(ip_addr, &(sin->sin_addr.s_addr), sizeof(*ip_addr));
+
+   pcap_freealldevs(alldevsp);
+
+   return 0;
+}
