@@ -1,5 +1,5 @@
 /*
- * ARP Scan is Copyright (C) 2005-2011 Roy Hills, NTA Monitor Ltd.
+ * ARP Scan is Copyright (C) 2005-2013 Roy Hills, NTA Monitor Ltd.
  *
  * This file is part of arp-scan.
  * 
@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with arp-scan.  If not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: arp-scan.h 18136 2011-02-25 21:29:45Z rsh $
+ * $Id: arp-scan.h 19575 2013-04-25 12:41:29Z rsh $
  *
  * arp-scan.h -- Header file for ARP scanner
  *
@@ -111,18 +111,6 @@
 #endif
 
 #ifdef HAVE_PCAP_H
-/*
- * The pcap.h header file on Apple Mac OS Xcode 2.5 and later includes pcap's
- * cut-down version of bpf.h, which defines macros that conflict with those in
- * the full bpf.h. To avoid the conflict, we include net/bpf.h before pcap.h
- * if compiling under Xcode 2.5 or later. This defines all the required macros
- * and prevents pcap's cut-down version from defining its own ones.
- *
- * 5370 is the value of __APPLE_CC__ for Xcode 2.5 on Tiger with GCC 4.0.1
- */
-#if defined(__APPLE_CC__) && (__APPLE_CC__ >= 5370)
-#include <net/bpf.h>
-#endif
 #include <pcap.h>
 #endif
 
@@ -148,7 +136,7 @@
 #define MINIMUM_FRAME_SIZE 46           /* Minimum layer 2 date size */
 #define DEFAULT_BACKOFF_FACTOR 1.5      /* Default timeout backoff factor */
 #define DEFAULT_RETRY 2                 /* Default number of retries */
-#define DEFAULT_TIMEOUT 100             /* Default per-host timeout in ms */
+#define DEFAULT_TIMEOUT 500             /* Default per-host timeout in ms */
 #define SNAPLEN 64			/* 14 (ether) + 28 (ARP) + extra */
 #define PROMISC 0			/* Enable promiscuous mode */
 #define TO_MS 0				/* Timeout for pcap_open_live() */
@@ -207,9 +195,6 @@ typedef struct {
    uint32_t ar_tip;		/* Target IP address */
 } arp_ether_ipv4;
 
-/* Link-layer handle structure, defined in link-xxx.c  */
-typedef struct link_handle link_t;
-
 /* Functions */
 
 #ifndef HAVE_STRLCAT
@@ -227,26 +212,26 @@ void err_print(int, const char *, va_list);
 void usage(int, int);
 void add_host_pattern(const char *, unsigned);
 void add_host(const char *, unsigned, int);
-int send_packet(link_t *, host_entry *, struct timeval *);
-void recvfrom_wto(int, int);
+int send_packet(pcap_t *, host_entry *, struct timeval *);
+void recvfrom_wto(int, int, pcap_t *);
 void remove_host(host_entry **);
 void timeval_diff(const struct timeval *, const struct timeval *,
                   struct timeval *);
 host_entry *find_host(host_entry **, struct in_addr *);
 void display_packet(host_entry *, arp_ether_ipv4 *, const unsigned char *,
-                    size_t, int, int, ether_hdr *);
+                    size_t, int, int, ether_hdr *, const struct pcap_pkthdr *);
 void advance_cursor(void);
 void dump_list(void);
 void print_times(void);
-void clean_up(void);
+void clean_up(pcap_t *);
 void arp_scan_version(void);
 char *make_message(const char *, ...);
 void callback(u_char *, const struct pcap_pkthdr *, const u_char *);
 void process_options(int, char *[]);
 struct in_addr *get_host_address(const char *, int, struct in_addr *, char **);
 const char *my_ntoa(struct in_addr);
-int get_source_ip(link_t *, uint32_t *);
-void get_hardware_address(link_t *, unsigned char []);
+int get_source_ip(const char *, uint32_t *);
+void get_hardware_address(const char *, unsigned char []);
 void marshal_arp_pkt(unsigned char *, ether_hdr *, arp_ether_ipv4 *, size_t *,
                      const unsigned char *, size_t);
 int unmarshal_arp_pkt(const unsigned char *, size_t, ether_hdr *,
@@ -256,10 +241,7 @@ unsigned int hstr_i(const char *);
 char *hexstring(const unsigned char *, size_t);
 int get_ether_addr(const char *, unsigned char *);
 int add_mac_vendor(struct hash_control *, const char *);
-/* Link layer send functions */
-link_t *link_open(const char *);
-ssize_t link_send(link_t *, const unsigned char *, size_t);
-void link_close(link_t *);
+char *get_mac_vendor_filename(const char *, const char *, const char *);
 /* Wrappers */
 int Gettimeofday(struct timeval *);
 void *Malloc(size_t);
